package dev.fitko.fitconnect.client.sender.model;

import dev.fitko.fitconnect.api.domain.model.metadata.AuthenticationInformation;
import dev.fitko.fitconnect.api.domain.model.metadata.payment.PaymentInformation;
import dev.fitko.fitconnect.api.domain.model.replychannel.ReplyChannel;
import dev.fitko.fitconnect.client.sender.steps.unencrypted.*;
import lombok.Getter;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.net.URI;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.UUID;

import static dev.fitko.fitconnect.api.domain.model.metadata.data.MimeType.APPLICATION_JSON;
import static dev.fitko.fitconnect.api.domain.model.metadata.data.MimeType.APPLICATION_XML;

@Getter
public class SendableSubmission {

    private final UUID destinationId;
    private final String data;
    private final String dataMimeType;
    private final List<Attachment> attachments;
    private final String serviceName;
    private final String serviceIdentifier;
    private final List<AuthenticationInformation> authenticationInformation;
    private final PaymentInformation paymentInformation;
    private final ReplyChannel replyChannel;
    private final URI schemaUri;

    private SendableSubmission(final Builder builder) {
        destinationId = builder.getDestinationId();
        data = builder.getData();
        dataMimeType = builder.getDataMimeType();
        attachments = Collections.unmodifiableList(builder.getAttachments());
        serviceName = builder.getServiceName();
        serviceIdentifier = builder.getServiceIdentifier();
        authenticationInformation = builder.getAuthenticationInformation();
        replyChannel = builder.getReplyChannel();
        paymentInformation = builder.getPaymentInformation();
        schemaUri = builder.getSchemaUri();
    }

    public static DestinationStep Builder() {
        return new Builder();
    }

    @Getter
    private static final class Builder implements DestinationStep,
            ServiceTypeStep,
            DataStep,
            OptionalPropertiesStep,
            BuildStep {

        private static final Logger LOGGER = LoggerFactory.getLogger(Builder.class);

        private UUID destinationId;
        private final List<Attachment> attachments = new ArrayList<>();
        private String data;
        private String dataMimeType;

        private String serviceName;

        private String serviceIdentifier;

        private List<AuthenticationInformation> authenticationInformation;

        private PaymentInformation paymentInformation;

        private ReplyChannel replyChannel;
        private URI schemaUri;

        private Builder() {
        }

        @Override
        public ServiceTypeStep setDestination(final UUID destinationId) {
            this.destinationId = destinationId;
            return this;
        }

        @Override
        public OptionalPropertiesStep addAttachments(final List<Attachment> attachments) {
            if (attachments != null && !attachments.isEmpty()) {
                this.attachments.addAll(attachments);
            }
            return this;
        }

        @Override
        public OptionalPropertiesStep addAttachment(final Attachment attachment) {
            if (attachment != null) {
                attachments.add(attachment);
            }
            return this;
        }

        @Override
        public OptionalPropertiesStep setReplyChannel(final ReplyChannel replyChannel) {
            this.replyChannel = replyChannel;
            return this;
        }

        @Override
        public OptionalPropertiesStep setJsonData(final String jsonData, final URI schemaUri) {
            data = jsonData;
            dataMimeType = APPLICATION_JSON.value();
            this.schemaUri = schemaUri;
            return this;
        }


        @Override
        public OptionalPropertiesStep setXmlData(final String xmlData, final URI schemaUri) {
            data = xmlData;
            dataMimeType = APPLICATION_XML.value();
            return this;
        }

        @Override
        public DataStep setServiceType(final String serviceIdentifier, final String serviceName) {
            this.serviceIdentifier = serviceIdentifier;
            this.serviceName = serviceName;
            return this;
        }

        @Override
        public OptionalPropertiesStep setAuthenticationInformation(final List<AuthenticationInformation> authenticationInformation) {
            this.authenticationInformation = authenticationInformation;
            return this;
        }

        @Override
        public OptionalPropertiesStep setPaymentInformation(final PaymentInformation paymentInformation) {
            this.paymentInformation = paymentInformation;
            return this;
        }

        @Override
        public SendableSubmission build() {
            return new SendableSubmission(this);
        }
    }
}
